package cc.smtweb.biz.mztb.web.controller;

import cc.smtweb.biz.mztb.web.base.*;
import cc.smtweb.framework.core.common.R;
import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.EntityHelper;
import cc.smtweb.framework.core.util.DateUtil;
import cc.smtweb.framework.core.util.RandomUtil;
import cc.smtweb.framework.core.util.StringUtil;
import cc.smtweb.system.bpm.web.sys.user.dept.Dept;
import cc.smtweb.system.bpm.web.sys.user.dept.DeptCache;
import cc.smtweb.system.bpm.web.sys.user.party.Party;
import cc.smtweb.system.bpm.web.sys.user.party.PartyCache;
import cc.smtweb.system.bpm.web.sys.user.role.group.RoleGroup;
import cc.smtweb.system.bpm.web.sys.user.role.group.RoleGroupCache;
import cc.smtweb.system.bpm.web.sys.user.user.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 给北京民政端提供接口
 */
@Slf4j
@Controller
@RequestMapping("/api_c")
public class MztbController {

    private static final String PWD_SALT = "goodpj";

    /**
     * 民政端推送机构数据
     * @param swMap
     * @return
     */
    @PostMapping("/push")
    @ResponseBody
    public R pushWork(@RequestBody SwMap swMap) {
        log.info("收到民政端推送机构数据，请求参数："+swMap);
//        addNewPartyAccount();
        DbEngine instance = DbEngine.getInstance();
        log.info("传入isRet参数，视为删除机构数据");
        if (swMap.containsKey("isRet")){
            //删除养老机构
            String codes = swMap.readString("code");
            if (codes.isEmpty()){
                log.info("未传入需要删除的统一社会信用代码！");
                return R.error(2,"未获取到需要删除的统一社会信用代码!");
            }
            //先将所有需要删除的用户查出  然后根据id去删除其他相关表数据
            List<String> codeList = Arrays.stream(codes.split(",")).collect(Collectors.toList());
            List<String> code = new ArrayList<>();
            codeList.forEach(temp->{
                code.add("'"+temp+"'");
            });
            List<User> userList = instance.query("select * from " + EntityHelper.getSchemaTableName(User.ENTITY_NAME) + " where sur_nicky in (" + String.join(",",code)  + ")", User.class);
            if (userList.isEmpty()){
                log.info("查询失败！");
                return R.success();
            }
            List<Long> userIds = userList.stream().map(User::getId).collect(Collectors.toList());
            List<String>  ids = new ArrayList<>();
            userIds.forEach(temp->ids.add(temp+""));
            AtomicInteger num = new AtomicInteger();
            instance.doTrans(()->{
                //单位信息表
                instance.update("delete from " + EntityHelper.getSchemaTableName(PartyInfo.ENTITY_NAME) + " where pi_credit_code in (" +  String.join(",",code) + ")");
                //用户表
                 num.set(instance.findDao(User.class).batchDelete(userList));
            });

            if (num.get() != codeList.size()){
                log.info("删除失败！");
                List<User> users = instance.queryWhere(User.class, " sur_nicky in (" + codes + ")");
                String retCodes = users.stream().map(User::getNicky).collect(Collectors.joining(","));
                return R.error(3,retCodes);
            }
            log.info("删除成功！");
            return R.success();
        }


        /*****************非删除机构,而是同步机构数据************************/
        List<Map<String, Object>> paramData = swMap.readListMap("data");
        if (paramData == null || paramData.size() < 1) {
            log.info("请求携带需要更新的机构数据为空！");
            return R.success();
        }
        log.debug("获取到数据:"+paramData.size()+"条，准备处理机构数据");
        //已有的机构数据则更新、不存在的则新增，同时检测并添加不存在的账号，
        Map<String, PartyInfo> tbPartyInfoMapByCode = instance.query(PartyInfo.class).stream().collect(Collectors.toMap(PartyInfo::getCreditCode, Function.identity()));
        List<PartyInfo> updPartyInfoList = new ArrayList<>();
        List<PartyInfo> addPartyInfoList = new ArrayList<>();
        List<Party> addPartyList = new ArrayList<>();
        Set<String> addPartyIdSet = new HashSet<>();
        List<User> addUserList = new ArrayList<>();
        List<Dept> addDeptList = new ArrayList<>();
        List<UserParty> addUserPartyList = new ArrayList<>();
        List<UserRole> addUserRoleList = new ArrayList<>();
        List<UserStatu> addUserStatuList = new ArrayList<>();
        Set<String> orgCodeMap = new HashSet<>();
        for (Map<String, Object> orgInfo : paramData) {
            SwMap data = new SwMap();
            data.putAll(orgInfo);
            String orgCode = data.readString("pi_credit_code");
            if(StringUtil.isEmpty(orgCode) || orgCodeMap.contains(orgCode)){
                //一次机构 同一个信用代码的只接受一次
                log.info("信用代码为空或者重复，跳过处理！数据为："+orgInfo);
                continue;
            }else{
                orgCodeMap.add(orgCode);
            }
            //新增或同步机构数据
            PartyInfo partyInfo = new PartyInfo();
            partyInfo.init();
            partyInfo.getData().putAll(orgInfo);
            if(tbPartyInfoMapByCode.containsKey(orgCode)){
                //机构已存在，只更新数据
                partyInfo.setId(tbPartyInfoMapByCode.get(orgCode).getId());
                partyInfo.setSysPartyId(tbPartyInfoMapByCode.get(orgCode).getSysPartyId());
                updPartyInfoList.add(partyInfo);
            }else{
                partyInfo.setId(DbEngine.getInstance().nextId());
                addPartyInfoList.add(partyInfo);
            }

            //判断系统机构是否存在 ，不存在 则新增
            Party party = PartyCache.getInstance().getByCode(orgCode);
            if (party == null) {
                party = new Party(); // 底层机构表
                party.init();
                party.setId(partyInfo.getId());
                party.setCode(partyInfo.getCreditCode());
                party.setName(partyInfo.getPartyName());
                party.setType(101);
                party.setAreaId(Long.parseLong(partyInfo.getAreaCode()));
                addPartyList.add(party);
                addPartyIdSet.add(String.valueOf(party.getId()));
            }
            partyInfo.setSysPartyId(party.getId());
            //添加系统用户
            User user = UserCache.getInstance().getByCode(partyInfo.getCreditCode());
            if(user == null){
                user = new User();
                user.init();
                long userId = instance.nextId();
                user.setId(userId);
                user.setCode(partyInfo.getCreditCode());
                user.setName(partyInfo.getPartyName() + "-填报人员");
                user.setNicky(partyInfo.getCreditCode());
                user.setLevel(100);
                user.setStatu(1);
//                    user.setPwd("13217b04835b4cbceb55edb54a72deb4"); //默认密码 abc@123456
                user.setPwd(DigestUtils.md5Hex(PWD_SALT + user.getId() + "abc@123456")); //默认密码 abc@123456
                user.setPwdLevel(0);
                addUserList.add(user);

                Dept dept = new Dept();// 添加部门
                dept.init();
                long deptId = instance.nextId();
                dept.setId(deptId);
                dept.setCode(RandomUtil.randomString(8));
                dept.setName("填报部门");
                dept.setType("100");
                dept.setPtId(party.getId());
                dept.setStatu(0);
                addDeptList.add(dept);

                UserParty userParty = new UserParty(); //用户机构关联表
                long userPartyId = instance.nextId();
                userParty.setId(userPartyId);
                userParty.setPartyId(party.getId());
                userParty.setUserId(userId);
                userParty.setDeptId(deptId); // 关联部门表
                addUserPartyList.add(userParty);

                UserRole userRole = new UserRole(); // 用户角色
                userRole.setId(instance.nextId());
                userRole.setSupId(userPartyId);
                userRole.setRoleGroupId(927515588790587392L); // 角色组id todo 先写死 一般不会变
                addUserRoleList.add(userRole);

                UserStatu userStatu = new UserStatu(); // 用户登录状态
                userStatu.init();
                userStatu.setId(userId);
                addUserStatuList.add(userStatu);
            }
        }
        instance.doTrans(() -> {
            // 处理新增或者修改的数据
            instance.findDao(PartyInfo.class).batchInsertEntity(addPartyInfoList);
            instance.findDao(PartyInfo.class).batchUpdateEntity(updPartyInfoList);
            instance.findDao(Party.class).batchInsertEntity(addPartyList);
            instance.findDao(User.class).batchInsertEntity(addUserList);
            instance.findDao(UserParty.class).batchInsertEntity(addUserPartyList);
            instance.findDao(Dept.class).batchInsertEntity(addDeptList);
            instance.findDao(UserRole.class).batchInsertEntity(addUserRoleList);
            instance.findDao(UserStatu.class).batchInsertEntity(addUserStatuList);
            log.debug("数据写入完成-----");

            // 将新增的机构 加入角色组的 机构字段
            if (addPartyIdSet.size() > 0) {
                instance.update("UPDATE " + EntityHelper.getSchemaTableName(RoleGroup.ENTITY_NAME)
                        + " SET srg_party_id = CONCAT(srg_party_id,',' ,'" + String.join(",", addPartyIdSet) + "') where srg_id = ? ", 927515588790587392L); // 角色组填报员
            }
            // 将机构、用户信息放入缓存,或者刷新缓存
            PartyCache.getInstance().refresh();
            UserCache.getInstance().refresh();
            UserPartyCache.getInstance().refresh();
            DeptCache.getInstance().refresh();
            UserRoleCache.getInstance().refresh();
            UserStatuCache.getInstance().refresh();
            RoleGroupCache.getInstance().refresh();
            log.debug("机构缓存刷新完成！");
        });
        log.debug("数据写入完成-----,updPartyInfoList："+updPartyInfoList.size()
                +",addPartyInfoList："+addPartyInfoList.size()
                +",addPartyList："+addPartyList.size()
                +",addUserList："+addUserList.size()
                +",addUserPartyList："+addUserPartyList.size()
                +",addDeptList："+addDeptList.size()
                +",addUserRoleList："+addUserRoleList.size()
                +",addUserStatuList："+addUserStatuList.size());
        return R.success();
    }

    //民政端拉取填报数据
    @PostMapping("/pull")
    @ResponseBody
    public R pullWork(@RequestBody SwMap swMap) {
        DbEngine instance = DbEngine.getInstance();

        String data = swMap.readString("data");
        boolean rollBack = swMap.readBool("rollBack", false);
        if (rollBack) {
            List<String> successList = swMap.readListString("success");
            List<String> errorList = swMap.readListString("error");
            DbEngine.getInstance().doTrans(()->{
                instance.update("update " + EntityHelper.getSchemaTableName(FillOut.ENTITY_NAME)
                        + " set fo_read_state = 1,fo_state = 1 where fo_id in (" + StringUtil.join(errorList, ",") + ")");
                instance.update("update " + EntityHelper.getSchemaTableName(FillOut.ENTITY_NAME)
                        + " set fo_read_time = ?, fo_read_state = 0 ,fo_state = 2 where fo_id in (" + StringUtil.join(successList, ",") + ")", DateUtil.nowDateTimeLong());
            });

            return R.success();
        }

        // 查询已确认还未推送和未成功 并且为指定任务月的填报数据
        List<String> fillOutList = instance.queryStringList(
                "select fo_id from " + EntityHelper.getSchemaTableName(FillOut.ENTITY_NAME)
                        + " where fo_state = 1 and fo_read_state =-1 and fo_task in (" + data + ") ");
        if (fillOutList.size() == 0) {
            log.info("未找到还未推送的数据");
            return R.success();
        }
        String sql = "select pi.pi_sys_party_id,fo.*,fi.*,bi.*,yep.* from " + EntityHelper.getSchemaTableName(FillOut.ENTITY_NAME) + " fo\n" +
                "    inner join " + EntityHelper.getSchemaTableName(FinanceInfo.ENTITY_NAME) + " fi on fo_id = fi_fo_id\n" +
                "    inner join " + EntityHelper.getSchemaTableName(BusinessInfo.ENTITY_NAME) + " bi on fo_id = bi_fo_id\n" +
                "    inner join "+EntityHelper.getSchemaTableName(PartyInfo.ENTITY_NAME)+" pi on fo_party_id = pi_id " +
                "    left join " + EntityHelper.getSchemaTableName(YearEndPersons.ENTITY_NAME) + " yep on fo_id = yep_fo_id " +
                "    where fo_id in (" + StringUtil.join(fillOutList, ",") + ");";

        List<SwMap> query = instance.query(sql, SwMap.class);

        return R.success(query);
    }

    //数据驳回
    @PostMapping("/dataReject")
    @ResponseBody
    public R dataRejectWorker(@RequestBody SwMap swMap) {
        DbEngine instance = DbEngine.getInstance();
        List<String> data = swMap.readListString("data");
        String sql ="update "+EntityHelper.getSchemaTableName(FillOut.ENTITY_NAME)+" set fo_state=3,fo_read_state=-1  where fo_id in ("+String.join(",",data)+")";
        int update = instance.update(sql);
        if (data.size() != update){
            List<FillOut> fillOuts = instance.findDao(FillOut.class).queryWhere(" fo_id in (" + String.join(",", data) + ")");
            List<String> errorList = new ArrayList<>();
            for (FillOut fillOut : fillOuts) {
                if (fillOut.getState() == 2 && fillOut.getReadState() == 0){
                    errorList.add(fillOut.getId()+"");
                }
            }
            return R.error(1,String.join(",",errorList));
        }else {
            return R.success();
        }
    }




}